import {randomUserData, type UserData} from './rand';

const lpad = (value: string, charCount: number, padChar: string): string => {
  expect(padChar).to.have.length(1);
  let modifiedValue: string = value;
  while (modifiedValue.length < charCount) {
    modifiedValue = padChar + modifiedValue;
  }
  return modifiedValue;
};

export const createUser = (userData?: UserData): UserData => {
  cy.visit('/');
  cy.get('.listView').should('be.visible');
  cy.get('.uStandbyWrapper').should('have.length', 0);
  cy.get('.uGrid__header .uButton').eq(0).click();
  cy.get('.editView').should('be.visible');
  cy.get('.uStandbyWrapper').should('have.length', 0);
  const user = userData || randomUserData();
  cy.get('input[name="firstname"]').type(user.firstname, {scrollBehavior: 'center'});
  cy.get('input[name="lastname"]').type(user.lastname, {scrollBehavior: 'center'});
  cy.get('input[name="name"]').type(user.username, {scrollBehavior: 'center'});
  if (user.birthday) {
    const birthdayString = [
      lpad(String(user.birthday[0]), 2, '0'),
      lpad(String(user.birthday[1]), 2, '0'),
      lpad(String(user.birthday[2]), 2, '0'),
    ].join('-');
    cy.get('input[name="birthday"]').type(birthdayString, {scrollBehavior: 'center'});
  }
  if (user.email) {
    cy.get('input[name="primaryMail"]').type(user.email, {scrollBehavior: 'center'});
  }
  cy.get('input[name="school"]')
    .closest('.uFormElement')
    .within(() => {
      cy.get('.uButton')
        .click();
      cy.get('.uComboBox__list__option')
        .should('have.length.above', 1)
        .eq(1)
        .click();
    });
  cy.get('[name="schoolClasses"]')
    .closest('.uFormElement')
    .find('select')
    .select(2);
  cy.get('input[name="role"]')
    .closest('.uFormElement')
    .within(() => {
      cy.get('.uButton')
        .click();
      cy.get('.uComboBox__list__option')
        .should('have.length.above', 1)
        .eq(0)
        .click();
    });
  cy.get('.listView__header__buttons [type="button"]')
    .eq(0)
    .click();
  cy.url().should('eq', `${Cypress.config().baseUrl}/`);
  // the next two assertions make sure the tests work in firefox: Because
  //  firefox will cause the application to trigger a NetworkError if an
  //  on-going AJAX request is aborted. So we will wait for all AJAX
  //  requests to finish before considering this step to be finished.
  cy.get('.listView').should('be.visible');
  cy.get('.uStandbyWrapper').should('have.length', 0);
  return user;
};

export const createMultipleUsers = (users: number | UserData[]): UserData[] => {
  let usersList: UserData[] = [];
  if (typeof users === 'number') {
    for (let i = 0; i < users; i++) {
      usersList.push(randomUserData());
    }
  } else {
    usersList = users;
  }
  return usersList.map(createUser);
};
